@@ -13,7 +13,7 @@ Tools for plotting and controlling plot output using context managers
    BasicPlottingContext
    JupyterPlottingContext
    get_plotting_context
-   napari_viewer
+   napari_add_layers
 
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 """
@@ -23,40 +23,23 @@ import contextlib
 import functools
 import logging
 import warnings
-from typing import TYPE_CHECKING, Any, Dict, Tuple, Type  # @UnusedImport
-
-from mpl_toolkits import axes_grid1
+from typing import Type  # @UnusedImport
+from typing import TYPE_CHECKING, Any, Dict, Generator, Tuple
 
 from ..tools.docstrings import replace_in_docstring
 
 if TYPE_CHECKING:
+    import matplotlib.cm  # @UnusedImport
+    import napari  # @UnusedImport
+
     from ..grids.base import GridBase  # @UnusedImport
 
 
-class _AxesXY(axes_grid1.axes_size._Base):
-    """
-    Scaled size whose relative part corresponds to the maximum of the data width and
-    data height of the *axes* multiplied by the *aspect*.
-    """
-
-    def __init__(self, axes, aspect=1.0):
-        self._axes = axes
-        self._aspect = aspect
-
-    def get_size(self, renderer):
-        l1, l2 = self._axes.get_xlim()
-        rel_size_x = abs(l2 - l1) * self._aspect
-
-        l1, l2 = self._axes.get_ylim()
-        rel_size_y = abs(l2 - l1) * self._aspect
-
-        abs_size = 0.0
-        rel_size = max(rel_size_x, rel_size_y)
-        return rel_size, abs_size
-
-
 def add_scaled_colorbar(
-    axes_image, aspect: float = 20, pad_fraction: float = 0.5, **kwargs
+    axes_image: "matplotlib.cm.ScalarMappable",
+    aspect: float = 20,
+    pad_fraction: float = 0.5,
+    **kwargs,
 ):
     """add a vertical color bar to an image plot
 
@@ -68,20 +51,57 @@ def add_scaled_colorbar(
     Inspired by https://stackoverflow.com/a/33505522/932593
 
     Args:
-        axes_image: object returned from :meth:`matplotlib.pyplot.imshow`
-        ax (:class:`matplotlib.axes.Axes`): the current figure axes
-        aspect (float): the target aspect ratio of the colorbar
-        pad_fraction (float): Width of the gap between colorbar and image
-        **kwargs: Additional parameters are passed to colorbar call
+        axes_image (:class:`matplotlib.cm.ScalarMappable`):
+            Mappable object, e.g., returned from :meth:`matplotlib.pyplot.imshow`
+        ax (:class:`matplotlib.axes.Axes`):
+            The current figure axes from which space is taken for the colorbar
+        aspect (float):
+            The target aspect ratio of the colorbar
+        pad_fraction (float):
+            Width of the gap between colorbar and image
+        **kwargs:
+            Additional parameters are passed to colorbar call
 
     Returns:
-        the result of the colorbar call
+        :class:`~matplotlib.colorbar.Colorbar`: the resulting Colorbar object
     """
+
+    from mpl_toolkits import axes_grid1
+
+    class _AxesXY(axes_grid1.axes_size._Base):
+        """
+        Scaled size whose relative part corresponds to the maximum of the data width and
+        data height of the *axes* multiplied by the *aspect*.
+        """
+
+        def __init__(self, axes, aspect=1.0):
+            self._axes = axes
+            self._aspect = aspect
+
+        def get_size(self, renderer):
+            l1, l2 = self._axes.get_xlim()
+            rel_size_x = abs(l2 - l1) * self._aspect
+
+            l1, l2 = self._axes.get_ylim()
+            rel_size_y = abs(l2 - l1) * self._aspect
+
+            abs_size = 0.0
+            rel_size = max(rel_size_x, rel_size_y)
+            return rel_size, abs_size
+
+    # make space for the colorbar and generate its axes
     divider = axes_grid1.make_axes_locatable(axes_image.axes)
     width = _AxesXY(axes_image.axes, aspect=1.0 / aspect)
     pad = axes_grid1.axes_size.Fraction(pad_fraction, width)
     cax = divider.append_axes("right", size=width, pad=pad)
-    return axes_image.axes.figure.colorbar(axes_image, cax=cax, **kwargs)
+
+    # create the colorbar
+    cbar = axes_image.axes.figure.colorbar(axes_image, cax=cax, **kwargs)
+
+    # disable the offset that matplotlib sometimes shows
+    cax.get_xaxis().get_major_formatter().set_useOffset(False)
+    cax.get_yaxis().get_major_formatter().set_useOffset(False)
+    return cbar
 
 
 class nested_plotting_check:
@@ -281,7 +301,9 @@ def plot_on_axes(wrapped=None, update_method=None):
         ax_style (dict):
             Dictionary with properties that will be changed on the axis
             after the plot has been drawn by calling
-            :meth:`matplotlib.pyplot.setp`.
+            :meth:`matplotlib.pyplot.setp`. A special item in this dictionary is
+            `use_offset`, which is flag that can be used to control whether offset are
+            shown along the axes of the plot.
         fig_style (dict):
             Dictionary with properties that will be changed on the
             figure after the plot has been drawn by calling
@@ -296,6 +318,9 @@ def plot_on_axes(wrapped=None, update_method=None):
         import matplotlib as mpl
         import matplotlib.pyplot as plt
 
+        if ax_style is None:
+            ax_style = {}
+
         # some logic to check for nested plotting calls:
         with nested_plotting_check() as is_outermost_plot_call:
 
@@ -317,6 +342,11 @@ def plot_on_axes(wrapped=None, update_method=None):
                 # finishing touches...
                 if title is not None:
                     ax.set_title(title)
+
+                use_offset = ax_style.pop("use_offset", None)
+                if use_offset is not None:
+                    ax.get_xaxis().get_major_formatter().set_useOffset(use_offset)
+                    ax.get_yaxis().get_major_formatter().set_useOffset(use_offset)
                 if ax_style:
                     plt.setp(ax, **ax_style)
                 if fig_style:
@@ -766,21 +796,51 @@ def get_plotting_context(
 
 
 @contextlib.contextmanager
-def napari_viewer(grid: "GridBase", **kwargs):
+def napari_viewer(
+    grid: "GridBase", close=False, **kwargs
+) -> Generator["napari.viewer.Viewer", None, None]:
     """creates an napari viewer for interactive plotting
 
     Args:
         grid (:class:`pde.grids.base.GridBase`): The grid defining the space
+        close (bool): Whether to close the viewer immediately (e.g. for testing)
         **kwargs: Extra arguments are passed to :class:`napari.Viewer`
     """
-    import napari
+    import napari  # @Reimport
 
-    if grid.num_axes == 1:
-        raise RuntimeError("Interactive plotting needs at least 2 spatial dimensions")
+    # parse and set viewer arguments
+    kwargs.setdefault("axis_labels", grid.axes)
+    kwargs.setdefault("ndisplay", 3 if grid.num_axes >= 3 else 2)
 
-    viewer_args = kwargs
-    viewer_args.setdefault("axis_labels", grid.axes)
-    viewer_args.setdefault("ndisplay", 3 if grid.num_axes >= 3 else 2)
+    with napari.gui_qt() as app:  # create Qt GUI context
+        viewer = napari.Viewer(**kwargs)
 
-    with napari.gui_qt():  # create Qt GUI context
-        yield napari.Viewer(**viewer_args)
+        yield viewer
+
+        if close:
+            from qtpy.QtCore import QTimer
+
+            viewer.close()
+            QTimer().singleShot(100, app.quit)
+
+
+def napari_add_layers(
+    viewer: "napari.viewer.Viewer", layers_data: Dict[str, Dict[str, Any]]
+):
+    """adds layers to a `napari <http://napari.org/>`__ viewer
+
+    Args:
+        viewer (:class:`napar    i.viewer.Viewer`):
+            The napari application
+        layers_data (dict):
+            Data for all layers that will be added.
+    """
+    for name, layer_data in layers_data.items():
+        layer_data.setdefault("name", name)
+        layer_type = layer_data.pop("type")
+        try:
+            add_layer = getattr(viewer, f"add_{layer_type}")
+        except AttributeError:
+            raise RuntimeError(f"Unknown layer type: {layer_type}")
+        else:
+            add_layer(**layer_data)
